from django import template
from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.core.mail import send_mail
from django.core.urlresolvers import reverse
from django.utils.http import int_to_base36
from inviter.views import import_attribute, TOKEN_GENERATOR
import shortuuid


FROM_EMAIL = getattr(settings, 'INVITER_FROM_EMAIL', settings.DEFAULT_FROM_EMAIL)

token_generator = import_attribute(TOKEN_GENERATOR)

def send_invite(invitee, inviter, url = None, **kwargs):
    """
    Send the default invitation email assembled from 
    `inviter/email/subject.txt` and `inviter/email/body.txt`
    
    Both templates will receive all the optional arguments.
    
    :param invitee: The invited user
    :type invitee: User
    :param inviter: The inviting user
    :type inviter: User
    :param url: The invite URL
    """
    ctx = {'invitee': invitee, 'inviter': inviter}
    ctx.update(kwargs)
    ctx.update(site = Site.objects.get_current(), url = url)
    ctx = template.Context(ctx)
    
    subject = template.loader.get_template('inviter/email/subject.txt')
    body = template.loader.get_template('inviter/email/body.txt')
    
    subject = subject.render(ctx)
    body = body.render(ctx)
    
    subject = ' '.join(subject.split('\n'))
    
    send_mail(subject, body, FROM_EMAIL, [invitee.email])

def invite(email, inviter, sendfn = send_invite, **kwargs):
    """
    Invite a given email address and return a user model with the given email
    address.
    
    If the user does not exist yet, create one, give it a UUID username, mark
    it as inactive and send the email.
    
    If the user does exist, send another email.

    If the user does exist and is *not* inactive, do nothing.

    Returns the user object.
    
    To customize the sent email, pass in a custom sending function. The sending
    function will receive all the optional arguments.
    
    :param email: The email address
    :type email: str
    :param inviter: The user inviting the email address
    :type inviter: User model
    :param sendfn: An email sending function. Defaults to
        `inviter.utils.send_invite`
    :type sendfn: function
    :rtype: User
    """
    
    try:
        user = User.objects.get(email = email)
        if user.is_active:
            return user
    except User.DoesNotExist:
        user = User.objects.create(username = shortuuid.uuid(), email = email,
            is_active = False)
        user.set_password(User.objects.make_random_password())
        user.save()
    
    url_parts = int_to_base36(user.id), token_generator.make_token(user)
    url = reverse('inviter:register', args=url_parts)
    
    sendfn(user, inviter, url = url, **kwargs)
    return user
    


"""
This file demonstrates writing tests using the unittest module. These will pass
when you run "manage.py test".

Replace this with more appropriate tests for your application.
"""
from django.conf import settings
from django.contrib.auth.models import User
from django.core import mail
from django.core.urlresolvers import reverse
from django.test import TestCase
from django.utils.http import int_to_base36
from inviter.utils import invite, token_generator
import shortuuid

from django.contrib.auth import tests


class InviteTest(TestCase):
    def setUp(self):
        self.original_email_backend = settings.EMAIL_BACKEND
        settings.EMAIL_BACKEND = 'django.core.mail.backends.locmem.EmailBackend'
        self.inviter = User.objects.create(username = shortuuid.uuid())
        self.existing = User.objects.create(username = shortuuid.uuid(),
            email = 'existing@example.com')
        
    def tearDown(self):
        settings.EMAIL_BACKEND = self.original_email_backend

    def test_inviting(self):
        user = invite("foo@example.com", self.inviter)        
        self.assertFalse(user.is_active)
        self.assertEqual(1, len(mail.outbox))
        self.assertEqual(3, User.objects.count())
        
        user = invite("foo@example.com", self.inviter)
        self.assertFalse(user.is_active)
        self.assertEqual(2, len(mail.outbox))
        self.assertEqual(3, User.objects.count())
        
        user = invite("existing@example.com", self.inviter)
        self.assertTrue(user.is_active)
        self.assertEqual(2, len(mail.outbox))
        self.assertEqual(3, User.objects.count())
        
    def test_views(self):
        user = invite("foo@example.com", self.inviter)
        url_parts = int_to_base36(user.id), token_generator.make_token(user)
        
        url = reverse('inviter:register', args = url_parts)
        
        resp = self.client.get(url)
        
        self.assertEqual(200, resp.status_code, resp.status_code)
        
        resp = self.client.post(url, {'username': 'testuser', 'email': 'foo@example.com',
            'new_password1': 'test-1234', 'new_password2': 'test-1234'})
        
        self.assertEqual(302, resp.status_code, resp.content)
        
        self.client.login(username = 'testuser', password = 'test-1234')
        
        resp = self.client.get(reverse('inviter:done'))

        self.assertEqual(200, resp.status_code, resp.status_code)        
        
        




from django.conf.urls.defaults import patterns, url
from inviter.views import Register, Done

urlpatterns = patterns('',
    url(r'^done/$', Done.as_view(), name = 'done'),
    url(r'^register/(?P<uidb36>[0-9A-Za-z]{1,13})-(?P<token>[0-9A-Za-z]+)/$',
        Register.as_view(), name = 'register'),
)


from django import forms
from django.contrib.auth.models import User
from django.utils.translation import ugettext as _


class RegistrationForm(forms.ModelForm):
    new_password1 = forms.CharField(label=_("New password"), widget=forms.PasswordInput)
    new_password2 = forms.CharField(label=_("New password confirmation"), widget=forms.PasswordInput)

    class Meta:
        model = User
        fields = ('username', 'email')
        
    def __init__(self, *args, **kwargs):
        super(RegistrationForm, self).__init__(*args,**kwargs)
        # Hide the UUID string
        self.initial['username'] = ''

    def clean_new_password2(self):
        password1 = self.cleaned_data.get('new_password1')
        password2 = self.cleaned_data.get('new_password2')
        if password1 and password2:
            if password1 != password2:
                raise forms.ValidationError(_("The two password fields didn't match."))
        return password2

    def save(self, *args, **kwargs):
        user = super(RegistrationForm, self).save(*args,**kwargs)
        user.set_password(self.cleaned_data.get('new_password1'))
        user.is_active = True
        if kwargs.get('commit'):
            user.save()
        return user
    
    


# Create your views here.
from django.conf import settings
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.http import Http404, HttpResponseRedirect, HttpResponseForbidden
from django.utils import importlib
from django.utils.functional import LazyObject
from django.utils.http import base36_to_int
from django.views.generic.base import TemplateView


FORM = getattr(settings, 'INVITER_FORM', 'inviter.forms.RegistrationForm')
REDIRECT = getattr(settings, 'INVITER_REDIRECT', 'inviter:done')
TOKEN_GENERATOR = getattr(settings, 'INVITER_TOKEN_GENERATOR', 'inviter.tokens.generator')

def import_attribute(path):
    """
    Import an attribute from a module.
    """
    module = '.'.join(path.split('.')[:-1])
    function = path.split('.')[-1]

    module = importlib.import_module(module)
    return getattr(module, function)

class Register(TemplateView):
    """
    A registration view for invited users. The user model already exists - this
    view just takes care of setting a password and username, and maybe update
    the email address. Anywho - one can customize the form that is used.
    
    """
    template_name = 'inviter/register.html'
    form = import_attribute(FORM)
    token_generator = import_attribute(TOKEN_GENERATOR)
    redirect_url = REDIRECT
    
    def get_user(self, uidb36):
        try:
            uid_int = base36_to_int(uidb36) 
            user = User.objects.get(id = uid_int)
        except (ValueError, User.DoesNotExist):
            raise Http404("No such invited user.")
        return user

    def get(self, request, uidb36, token):
        """
        Unfortunately just a copy of 
        ``django.contrib.auth.views.password_reset_confirm``
        """
        assert uidb36 is not None and token is not None
        user = self.get_user(uidb36)

        if not self.token_generator.check_token(user, token):
            return HttpResponseForbidden()
        
        return self.render_to_response({'invitee': user, 'form': self.form(instance=user)})
        
    def post(self, request, uidb36, token):
        """
        Unfortunately just a copy of 
        ``django.contrib.auth.views.password_reset_confirm``        
        """
        assert uidb36 is not None and token is not None
        user = self.get_user(uidb36)
        
        if not self.token_generator.check_token(user, token):
            return HttpResponseForbidden()
        
        form = self.form(request.POST, instance = user)
        
        if form.is_valid():
            form.save()
            try:
                return HttpResponseRedirect(reverse(self.redirect_url))
            except:
                return HttpResponseRedirect(self.redirect_url)
        return self.render_to_response({'invitee': user, 'form': form})

class Done(TemplateView):
    template_name = 'inviter/done.html'
    
    def get(self, request):
        return self.render_to_response({})
    



from django.conf import settings
from django.utils.crypto import salted_hmac, constant_time_compare

class TokenGenerator(object):
    def make_token(self, user):
        return self._make_token(user)
    
    def _make_token(self, user):
        value = u'-'.join([unicode(user.id), unicode(user.last_login),
            user.password])
        return salted_hmac(settings.SECRET_KEY, value).hexdigest()[::2]
    
    def check_token(self, user, token):
        return constant_time_compare(token, self._make_token(user))
    
generator = TokenGenerator()


#!/usr/bin/env python

from setuptools import setup, find_packages
import inviter

setup(
    name='django-inviter',
    version=inviter.__version__,
    description='Invite users to your Django apps',
    long_description=open('README.rst').read(),
    author='Alen Mujezinovic',
    author_email='alen@caffeinehit.com',
    packages= filter(
        lambda package: not package.startswith('testproject'), find_packages()),
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ],
    install_requires=[
        "shortuuid >= 0.1",
    ],
    include_package_data=True,
    zip_safe=False,
)
